@@ -1,57 +1,57 @@
-# Copyright (C) 2020-2021 Sebastian Blauth
+# Copyright (C) 2020-2022 Sebastian Blauth
 #
-# This file is part of CASHOCS.
+# This file is part of cashocs.
 #
-# CASHOCS is free software: you can redistribute it and/or modify
+# cashocs is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
-# CASHOCS is distributed in the hope that it will be useful,
+# cashocs is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with CASHOCS.  If not, see <https://www.gnu.org/licenses/>.
+# along with cashocs.  If not, see <https://www.gnu.org/licenses/>.
 
-"""
-module for equality and inequality constraints
-"""
+"""Module for equality and inequality constraints."""
 
 from __future__ import annotations
 
 import abc
-from typing import Union, Optional
+from typing import Dict, Optional, Union
 
 import fenics
 import numpy as np
 import ufl
 import ufl.core.expr
 
-from .._exceptions import InputError
-from ..utils import _max, _min
+from cashocs import _exceptions
+from cashocs import _utils
 
 
 class Constraint(abc.ABC):
     """Base class for additional equality and inequality constraints."""
 
+    linear_term: Union[ufl.Form, ufl.core.expr.Expr]
+    quadratic_term: Union[ufl.Form, Dict]
+
     def __init__(
         self,
         variable_function: Union[ufl.Form, ufl.core.expr.Expr],
         measure: Optional[fenics.Measure] = None,
     ) -> None:
-        """
-        Parameters
-        ----------
-        variable_function : ufl.Form or ufl.core.expr.Expr
-            Either a ufl Form (when we have a scalar / integral constraint) or an ufl
-            expression (when we have a pointwise constraint), which models the part
-            that is to be constrained
-        measure : fenics.Measure or None, optional
-            A measure indicating where a pointwise constraint should be satisfied.
-        """
+        """Initializes self.
 
+        Args:
+            variable_function: Either a UFL Form (when we have a scalar / integral
+                constraint) or an ufl expression (when we have a pointwise constraint),
+                which models the part that is to be constrained.
+            measure: A measure indicating where a pointwise constraint should be
+                satisfied.
+
+        """
         self.variable_function = variable_function
         self.measure = measure
 
@@ -63,19 +63,14 @@ class Constraint(abc.ABC):
         else:
             self.is_pointwise_constraint = True
 
-        self.linear_term = None
-        self.quadratic_term = None
-
     @abc.abstractmethod
     def constraint_violation(self) -> float:
-        """Computes the constraint violation for self
+        """Computes the constraint violation for the problem.
 
-        Returns
-        -------
-        float
+        Returns:
             The computed violation
-        """
 
+        """
         pass
 
 
@@ -88,19 +83,17 @@ class EqualityConstraint(Constraint):
         target: float,
         measure: Optional[fenics.Measure] = None,
     ) -> None:
-        """
-        Parameters
-        ----------
-        variable_function : ufl.Form or ufl.core.expr.Expr
-            Either a ufl Form (when we have a scalar / integral constraint) or an ufl
-            expression (when we have a pointwise constraint), which models the part
-            that is to be constrained
-        target : float
-            The target (rhs) of the equality constraint.
-        measure : fenics.Measure or None, optional
-            A measure indicating where a pointwise constraint should be satisfied.
-        """
+        """Initializes self.
 
+        Args:
+            variable_function: Either a UFL Form (when we have a scalar / integral
+                constraint) or an ufl expression (when we have a pointwise constraint),
+                which models the part that is to be constrained.
+            target: The target (rhs) of the equality constraint.
+            measure: A measure indicating where a pointwise constraint should be
+                satisfied.
+
+        """
         super().__init__(variable_function, measure=measure)
         self.target = target
 
@@ -121,7 +114,13 @@ class EqualityConstraint(Constraint):
                 fenics.Constant(0.5) * pow(variable_function - target, 2) * measure
             )
 
-    def constraint_violation(self):
+    def constraint_violation(self) -> float:
+        """Computes the constraint violation for the problem.
+
+        Returns:
+            The computed violation
+
+        """
         if self.is_integral_constraint:
             return np.abs(fenics.assemble(self.variable_function) - self.target)
         elif self.is_pointwise_constraint:
@@ -130,37 +129,37 @@ class EqualityConstraint(Constraint):
                     pow(self.variable_function - self.target, 2) * self.measure
                 )
             )
+        return float("inf")
 
 
 class InequalityConstraint(Constraint):
+    """Models an (additional) inequality constraint."""
+
     def __init__(
         self,
         variable_function: Union[ufl.Form, ufl.core.expr.Expr],
         lower_bound: Optional[Union[float, fenics.Function]] = None,
         upper_bound: Optional[Union[float, fenics.Function]] = None,
         measure: Optional[fenics.Measure] = None,
-    ):
-        """
-        Parameters
-        ----------
-        variable_function : ufl.Form or ufl.core.expr.Expr
-            Either a ufl Form (when we have a scalar / integral constraint) or an ufl
-            expression (when we have a pointwise constraint), which models the part
-            that is to be constrained
-        lower_bound : float or fenics.Function or None, optional
-            The lower bound for the inequality constraint
-        upper_bound : float or fenics.Function or None, optional
-            The upper bound for the inequality constraint
-        measure : fenics.Measure or None, optional
-            A measure indicating where a pointwise constraint should be satisfied.
-        """
+    ) -> None:
+        """Initializes self.
 
+        Args:
+            variable_function: Either a UFL Form (when we have a scalar / integral
+                constraint) or an ufl expression (when we have a pointwise constraint),
+                which models the part that is to be constrained
+            lower_bound: The lower bound for the inequality constraint
+            upper_bound: The upper bound for the inequality constraint
+            measure: A measure indicating where a pointwise constraint should be
+                satisfied.
+
+        """
         super().__init__(variable_function, measure=measure)
         self.lower_bound = lower_bound
         self.upper_bound = upper_bound
 
         if self.lower_bound is None and self.upper_bound is None:
-            raise InputError(
+            raise _exceptions.InputError(
                 "cashocs._constraints.constraints.InequalityConstraint",
                 "lower_bound and upper_bound",
                 "You have to specify at least one bound for the inequality constraint.",
@@ -188,7 +187,7 @@ class InequalityConstraint(Constraint):
                     fenics.Constant(1 / 2)
                     / self.weight
                     * pow(
-                        _max(
+                        _utils.max_(
                             fenics.Constant(0.0),
                             self.multiplier
                             + self.weight * (self.variable_function - self.upper_bound),
@@ -203,7 +202,7 @@ class InequalityConstraint(Constraint):
                     fenics.Constant(1 / 2)
                     / self.weight
                     * pow(
-                        _min(
+                        _utils.min_(
                             fenics.Constant(0.0),
                             self.multiplier
                             + self.weight * (self.variable_function - self.lower_bound),
@@ -214,14 +213,12 @@ class InequalityConstraint(Constraint):
                 )
 
     def constraint_violation(self) -> float:
-        """Computes the constraint violation of self
+        """Computes the constraint violation for the problem.
+
+        Returns:
+            The computed violation
 
-        Returns
-        -------
-        float
-            The computed constraint violation.
         """
-
         violation = 0.0
         if self.is_integral_constraint:
             min_max_integral = fenics.assemble(self.min_max_term["integrand"])
@@ -241,7 +238,7 @@ class InequalityConstraint(Constraint):
             if self.upper_bound is not None:
                 violation += fenics.assemble(
                     pow(
-                        _max(
+                        _utils.max_(
                             self.variable_function - self.upper_bound,
                             fenics.Constant(0.0),
                         ),
@@ -253,7 +250,7 @@ class InequalityConstraint(Constraint):
             if self.lower_bound is not None:
                 violation += fenics.assemble(
                     pow(
-                        _min(
+                        _utils.min_(
                             self.variable_function - self.lower_bound,
                             fenics.Constant(0.0),
                         ),
