@@ -1,20 +1,21 @@
-# Copyright (C) 2020-2021 Sebastian Blauth
+# Copyright (C) 2020-2022 Sebastian Blauth
 #
-# This file is part of CASHOCS.
+# This file is part of cashocs.
 #
-# CASHOCS is free software: you can redistribute it and/or modify
+# cashocs is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
-# CASHOCS is distributed in the hope that it will be useful,
+# cashocs is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with CASHOCS.  If not, see <https://www.gnu.org/licenses/>.
+# along with cashocs.  If not, see <https://www.gnu.org/licenses/>.
 
+"""Module for managing UFL forms for optimal control problems."""
 
 from __future__ import annotations
 
@@ -22,45 +23,34 @@ from typing import List, TYPE_CHECKING
 
 import fenics
 import numpy as np
-from ufl import replace
-
-from .form_handler import FormHandler
-from .._exceptions import (
-    InputError,
-)
-from ..utils import (
-    summation,
-    _max,
-    _min,
-)
+import ufl
 
+from cashocs import _exceptions
+from cashocs import _utils
+from cashocs._forms import form_handler
 
 if TYPE_CHECKING:
-    from .._optimal_control.optimal_control_problem import OptimalControlProblem
+    from cashocs._optimization import optimal_control
 
 
-class ControlFormHandler(FormHandler):
+class ControlFormHandler(form_handler.FormHandler):
     """Class for UFL form manipulation for optimal control problems.
 
     This is used to symbolically derive the corresponding weak forms of the
     adjoint and gradient equations (via UFL) , that are later used in the
     solvers for the equations later on. These are needed as subroutines for
      the optimization (solution) algorithms.
-
-    See Also
-    --------
-    ShapeFormHandler : Derives the adjoint equations and shape derivatives for shape optimization problems
     """
 
-    def __init__(self, optimization_problem: OptimalControlProblem) -> None:
-        """Initializes the ControlFormHandler class.
+    def __init__(
+        self, optimization_problem: optimal_control.OptimalControlProblem
+    ) -> None:
+        """Initializes self.
+
+        Args:
+            optimization_problem: The corresponding optimal control problem
 
-        Parameters
-        ----------
-        optimization_problem : OptimalControlProblem
-            The corresponding optimal control problem
         """
-
         super().__init__(optimization_problem)
 
         # Initialize the attributes from the arguments
@@ -71,50 +61,66 @@ class ControlFormHandler(FormHandler):
             optimization_problem.require_control_constraints
         )
 
+        self.idx_active = None
+        self.idx_active_lower = None
+        self.idx_active_upper = None
+        self.idx_inactive = None
+
+        self.w_1 = None
+        self.w_2 = None
+        self.w_3 = None
+        self.hessian_rhs = None
+
         self.control_dim = len(self.controls)
         self.control_spaces = [x.function_space() for x in self.controls]
 
-        self.gradient = [fenics.Function(V) for V in self.control_spaces]
+        self.gradient = [
+            fenics.Function(function_space) for function_space in self.control_spaces
+        ]
 
         # Define the necessary functions
-        self.states_prime = [fenics.Function(V) for V in self.state_spaces]
-        self.adjoints_prime = [fenics.Function(V) for V in self.adjoint_spaces]
+        self.states_prime = [
+            fenics.Function(function_space) for function_space in self.state_spaces
+        ]
+        self.adjoints_prime = [
+            fenics.Function(function_space) for function_space in self.adjoint_spaces
+        ]
 
-        self.test_directions = [fenics.Function(V) for V in self.control_spaces]
+        self.test_directions = [
+            fenics.Function(function_space) for function_space in self.control_spaces
+        ]
 
         self.trial_functions_control = [
-            fenics.TrialFunction(V) for V in self.control_spaces
+            fenics.TrialFunction(function_space)
+            for function_space in self.control_spaces
         ]
         self.test_functions_control = [
-            fenics.TestFunction(V) for V in self.control_spaces
+            fenics.TestFunction(function_space)
+            for function_space in self.control_spaces
         ]
 
-        self.temp = [fenics.Function(V) for V in self.control_spaces]
+        self.temp = [
+            fenics.Function(function_space) for function_space in self.control_spaces
+        ]
 
         # Compute the necessary equations
-        self.__compute_gradient_equations()
+        self._compute_gradient_equations()
 
-        if self.opt_algo == "newton" or (
-            self.opt_algo == "pdas" and self.inner_pdas == "newton"
-        ):
-            self.__compute_newton_forms()
+        if self.opt_algo.casefold() == "newton":
+            self.compute_newton_forms()
 
         # Initialize the scalar products
         fenics_scalar_product_matrices = [
-            fenics.PETScMatrix() for i in range(self.control_dim)
+            fenics.PETScMatrix() for _ in range(self.control_dim)
         ]
-        [
+
+        for i in range(self.control_dim):
             fenics.assemble(
                 self.riesz_scalar_products[i],
                 keep_diagonal=True,
                 tensor=fenics_scalar_product_matrices[i],
             )
-            for i in range(self.control_dim)
-        ]
-        [
             fenics_scalar_product_matrices[i].ident_zeros()
-            for i in range(self.control_dim)
-        ]
         self.riesz_projection_matrices = [
             fenics_scalar_product_matrices[i].mat() for i in range(self.control_dim)
         ]
@@ -131,7 +137,7 @@ class ControlFormHandler(FormHandler):
                         / self.riesz_projection_matrices[i].norm()
                         < 1e-15
                     ):
-                        raise InputError(
+                        raise _exceptions.InputError(
                             "cashocs._forms.ControlFormHandler",
                             "riesz_scalar_products",
                             "Supplied scalar product form is not symmetric.",
@@ -142,19 +148,14 @@ class ControlFormHandler(FormHandler):
     ) -> float:
         """Computes the scalar product between control type functions a and b.
 
-        Parameters
-        ----------
-        a : list[fenics.Function]
-            The first argument.
-        b : list[fenics.Function]
-            The second argument.
+        Args:
+            a: The first argument.
+            b: The second argument.
 
-        Returns
-        -------
-        float
+        Returns:
             The value of the scalar product.
-        """
 
+        """
         result = 0.0
 
         for i in range(self.control_dim):
@@ -168,13 +169,7 @@ class ControlFormHandler(FormHandler):
         return result
 
     def compute_active_sets(self) -> None:
-        """Computes the indices corresponding to active and inactive sets.
-
-        Returns
-        -------
-        None
-        """
-
+        """Computes the indices corresponding to active and inactive sets."""
         self.idx_active_lower = []
         self.idx_active_upper = []
         self.idx_active = []
@@ -184,16 +179,20 @@ class ControlFormHandler(FormHandler):
 
             if self.require_control_constraints[j]:
                 self.idx_active_lower.append(
-                    (
-                        self.controls[j].vector()[:]
-                        <= self.control_constraints[j][0].vector()[:]
-                    ).nonzero()[0]
+                    np.nonzero(
+                        (
+                            self.controls[j].vector()[:]
+                            <= self.control_constraints[j][0].vector()[:]
+                        )
+                    )[0]
                 )
                 self.idx_active_upper.append(
-                    (
-                        self.controls[j].vector()[:]
-                        >= self.control_constraints[j][1].vector()[:]
-                    ).nonzero()[0]
+                    np.nonzero(
+                        (
+                            self.controls[j].vector()[:]
+                            >= self.control_constraints[j][1].vector()[:]
+                        )
+                    )[0]
                 )
             else:
                 self.idx_active_lower.append([])
@@ -215,22 +214,18 @@ class ControlFormHandler(FormHandler):
     ) -> List[fenics.Function]:
         """Restricts a function to the active set.
 
-        Restricts a control type function a onto the active set,
-        which is returned via the function b,  i.e., b is zero on the inactive set.
+        Restricts a control type function ``a`` onto the active set,
+        which is returned via the function ``b``,  i.e., ``b`` is zero on the inactive
+        set.
 
-        Parameters
-        ----------
-        a : list[fenics.Function]
-            The first argument, to be projected onto the active set.
-        b : list[fenics.Function]
-            The second argument, which stores the result (is overwritten).
+        Args:
+            a: The first argument, to be projected onto the active set.
+            b: The second argument, which stores the result (is overwritten).
 
-        Returns
-        -------
-        b : list[fenics.Function]
+        Returns:
             The result of the projection (overwrites input b).
-        """
 
+        """
         for j in range(self.control_dim):
             if self.require_control_constraints[j]:
                 self.temp[j].vector().vec().set(0.0)
@@ -244,91 +239,23 @@ class ControlFormHandler(FormHandler):
 
         return b
 
-    def restrict_to_lower_active_set(
-        self, a: List[fenics.Function], b: List[fenics.Function]
-    ) -> List[fenics.Function]:
-        """Restricts a function to the lower bound of the constraints
-
-        Parameters
-        ----------
-        a : list[fenics.Function]
-            The input, which is to be restricted
-        b : list[fenics.Function]
-            The output, which stores the result
-
-        Returns
-        -------
-        b : list[fenics.Function]
-            Function a restricted onto the lower boundaries of the constraints
-
-        """
-
-        for j in range(self.control_dim):
-            if self.require_control_constraints[j]:
-                self.temp[j].vector().vec().set(0.0)
-                self.temp[j].vector()[self.idx_active_lower[j]] = a[j].vector()[
-                    self.idx_active_lower[j]
-                ]
-                b[j].vector().vec().aypx(0.0, self.temp[j].vector().vec())
-
-            else:
-                b[j].vector().vec().set(0.0)
-
-        return b
-
-    def restrict_to_upper_active_set(
-        self, a: List[fenics.Function], b: List[fenics.Function]
-    ) -> List[fenics.Function]:
-        """Restricts a function to the upper bound of the constraints
-
-        Parameters
-        ----------
-        a : list[fenics.Function]
-            The input, which is to be restricted
-        b : list[fenics.Function]
-            The output, which stores the result
-
-        Returns
-        -------
-        b : list[fenics.Function]
-            Function a restricted onto the upper boundaries of the constraints
-
-        """
-
-        for j in range(self.control_dim):
-            if self.require_control_constraints[j]:
-                self.temp[j].vector().vec().set(0.0)
-                self.temp[j].vector()[self.idx_active_upper[j]] = a[j].vector()[
-                    self.idx_active_upper[j]
-                ]
-                b[j].vector().vec().aypx(0.0, self.temp[j].vector().vec())
-
-            else:
-                b[j].vector().vec().set(0.0)
-
-        return b
-
     def restrict_to_inactive_set(
         self, a: List[fenics.Function], b: List[fenics.Function]
     ) -> List[fenics.Function]:
         """Restricts a function to the inactive set.
 
-        Restricts a control type function a onto the inactive set,
-        which is returned via the function b, i.e., b is zero on the active set.
+        Restricts a control type function ``a`` onto the inactive set,
+        which is returned via the function ``b``, i.e., ``b`` is zero on the active set.
 
-        Parameters
-        ----------
-        a : list[fenics.Function]
-            The control-type function that is to be projected onto the inactive set.
-        b : list[fenics.Function]
-            The storage for the result of the projection (is overwritten).
+        Args:
+            a: The control-type function that is to be projected onto the inactive set.
+            b: The storage for the result of the projection (is overwritten).
+
+        Returns:
+            The result of the projection of ``a`` onto the inactive set (overwrites
+            input ``b``).
 
-        Returns
-        -------
-        b : list[fenics.Function]
-            The result of the projection of a onto the inactive set (overwrites input b).
         """
-
         for j in range(self.control_dim):
             if self.require_control_constraints[j]:
                 self.temp[j].vector().vec().set(0.0)
@@ -348,21 +275,17 @@ class ControlFormHandler(FormHandler):
     ) -> List[fenics.Function]:
         """Project a function to the set of admissible controls.
 
-        Projects a control type function a onto the set of admissible controls
+        Projects a control type function ``a`` onto the set of admissible controls
         (given by the box constraints).
 
-        Parameters
-        ----------
-        a : list[fenics.Function]
-            The function which is to be projected onto the set of admissible
-            controls (is overwritten)
+        Args:
+            a: The function which is to be projected onto the set of admissible
+                controls (is overwritten)
+
+        Returns:
+            The result of the projection (overwrites input ``a``)
 
-        Returns
-        -------
-        a : list[fenics.Function]
-            The result of the projection (overwrites input a)
         """
-
         for j in range(self.control_dim):
             if self.require_control_constraints[j]:
                 a[j].vector().vec().pointwiseMin(
@@ -374,14 +297,8 @@ class ControlFormHandler(FormHandler):
 
         return a
 
-    def __compute_gradient_equations(self) -> None:
-        """Calculates the variational form of the gradient equation, for the Riesz projection.
-
-        Returns
-        -------
-        None
-        """
-
+    def _compute_gradient_equations(self) -> None:
+        """Calculates the variational form of the gradient equation."""
         self.gradient_forms_rhs = [
             fenics.derivative(
                 self.lagrangian_form,
@@ -415,7 +332,7 @@ class ControlFormHandler(FormHandler):
                             self.min_max_integrand_values[j]
                             - self.min_max_lower_bounds[j]
                         )
-                        self.gradient_forms_rhs[i] += _min(
+                        self.gradient_forms_rhs[i] += _utils.min_(
                             fenics.Constant(0.0), term_lower
                         ) * fenics.derivative(
                             self.min_max_integrands[j],
@@ -428,7 +345,7 @@ class ControlFormHandler(FormHandler):
                             self.min_max_integrand_values[j]
                             - self.min_max_upper_bounds[j]
                         )
-                        self.gradient_forms_rhs[i] += _max(
+                        self.gradient_forms_rhs[i] += _utils.max_(
                             fenics.Constant(0.0), term_upper
                         ) * fenics.derivative(
                             self.min_max_integrands[j],
@@ -436,24 +353,11 @@ class ControlFormHandler(FormHandler):
                             self.test_functions_control[i],
                         )
 
-    def __compute_newton_forms(self) -> None:
-        """Calculates the needed forms for the truncated Newton method.
-
-        Returns
-        -------
-        None
-        """
-
-        if self.use_scalar_tracking or self.use_min_max_terms:
-            raise InputError(
-                "cashocs._forms.ShapeFormHandler",
-                "__compute_newton_forms",
-                "Newton's method is not available with scalar tracking or min_max terms.",
-            )
-
+    def _compute_sensitivity_equations(self) -> None:
+        """Calculates the forms for the (forward) sensitivity equations."""
         # Use replace -> derivative to speed up the computations
         self.sensitivity_eqs_temp = [
-            replace(
+            ufl.replace(
                 self.state_forms[i], {self.adjoints[i]: self.test_functions_state[i]}
             )
             for i in range(self.state_dim)
@@ -477,8 +381,9 @@ class ControlFormHandler(FormHandler):
 
         # Need to distinguish cases due to empty sum in case state_dim = 1
         if self.state_dim > 1:
+            # pylint: disable=invalid-unary-operand-type
             self.sensitivity_eqs_rhs = [
-                -summation(
+                -_utils.summation(
                     [
                         fenics.derivative(
                             self.sensitivity_eqs_temp[i],
@@ -489,7 +394,7 @@ class ControlFormHandler(FormHandler):
                         if j != i
                     ]
                 )
-                - summation(
+                - _utils.summation(
                     [
                         fenics.derivative(
                             self.sensitivity_eqs_temp[i],
@@ -503,7 +408,8 @@ class ControlFormHandler(FormHandler):
             ]
         else:
             self.sensitivity_eqs_rhs = [
-                -summation(
+                # pylint: disable=invalid-unary-operand-type
+                -_utils.summation(
                     [
                         fenics.derivative(
                             self.sensitivity_eqs_temp[i],
@@ -521,8 +427,9 @@ class ControlFormHandler(FormHandler):
             for i in range(self.state_dim):
                 self.sensitivity_eqs_picard[i] -= self.sensitivity_eqs_rhs[i]
 
-        # Compute forms for the truncated Newton method
-        self.L_y = [
+    def _compute_first_order_lagrangian_derivatives(self) -> None:
+        """Computes the derivative of the Lagrangian w.r.t. the state and control."""
+        self.lagrangian_y = [
             fenics.derivative(
                 self.lagrangian_form,
                 self.states[i],
@@ -530,7 +437,7 @@ class ControlFormHandler(FormHandler):
             )
             for i in range(self.state_dim)
         ]
-        self.L_u = [
+        self.lagrangian_u = [
             fenics.derivative(
                 self.lagrangian_form,
                 self.controls[i],
@@ -539,53 +446,50 @@ class ControlFormHandler(FormHandler):
             for i in range(self.control_dim)
         ]
 
-        self.L_yy = [
+    def _compute_second_order_lagrangian_derivatives(self) -> None:
+        """Compute the second order derivatives of the Lagrangian w.r.t. y and u."""
+        self.lagrangian_yy = [
             [
-                fenics.derivative(self.L_y[i], self.states[j], self.states_prime[j])
+                fenics.derivative(
+                    self.lagrangian_y[i], self.states[j], self.states_prime[j]
+                )
                 for j in range(self.state_dim)
             ]
             for i in range(self.state_dim)
         ]
-        self.L_yu = [
+        self.lagrangian_yu = [
             [
-                fenics.derivative(self.L_u[i], self.states[j], self.states_prime[j])
+                fenics.derivative(
+                    self.lagrangian_u[i], self.states[j], self.states_prime[j]
+                )
                 for j in range(self.state_dim)
             ]
             for i in range(self.control_dim)
         ]
-        self.L_uy = [
+        self.lagrangian_uy = [
             [
                 fenics.derivative(
-                    self.L_y[i], self.controls[j], self.test_directions[j]
+                    self.lagrangian_y[i], self.controls[j], self.test_directions[j]
                 )
                 for j in range(self.control_dim)
             ]
             for i in range(self.state_dim)
         ]
-        self.L_uu = [
+        self.lagrangian_uu = [
             [
                 fenics.derivative(
-                    self.L_u[i], self.controls[j], self.test_directions[j]
+                    self.lagrangian_u[i], self.controls[j], self.test_directions[j]
                 )
                 for j in range(self.control_dim)
             ]
             for i in range(self.control_dim)
         ]
 
-        self.w_1 = [
-            summation([self.L_yy[i][j] for j in range(self.state_dim)])
-            + summation([self.L_uy[i][j] for j in range(self.control_dim)])
-            for i in range(self.state_dim)
-        ]
-        self.w_2 = [
-            summation([self.L_yu[i][j] for j in range(self.state_dim)])
-            + summation([self.L_uu[i][j] for j in range(self.control_dim)])
-            for i in range(self.control_dim)
-        ]
-
+    def _compute_adjoint_sensitivity_equations(self) -> None:
+        """Computes the adjoint sensitivity equations for the Newton method."""
         # Use replace -> derivative for faster computations
         self.adjoint_sensitivity_eqs_diag_temp = [
-            replace(
+            ufl.replace(
                 self.state_forms[i], {self.adjoints[i]: self.trial_functions_adjoint[i]}
             )
             for i in range(self.state_dim)
@@ -595,7 +499,8 @@ class ControlFormHandler(FormHandler):
             self.adjoints[j]: self.adjoints_prime[j] for j in range(self.state_dim)
         }
         self.adjoint_sensitivity_eqs_all_temp = [
-            replace(self.state_forms[i], mapping_dict) for i in range(self.state_dim)
+            ufl.replace(self.state_forms[i], mapping_dict)
+            for i in range(self.state_dim)
         ]
 
         self.adjoint_sensitivity_eqs_lhs = [
@@ -619,7 +524,7 @@ class ControlFormHandler(FormHandler):
         # Need cases distinction due to empty sum for state_dim == 1
         if self.state_dim > 1:
             for i in range(self.state_dim):
-                self.w_1[i] -= summation(
+                self.w_1[i] -= _utils.summation(
                     [
                         fenics.derivative(
                             self.adjoint_sensitivity_eqs_all_temp[j],
@@ -639,7 +544,7 @@ class ControlFormHandler(FormHandler):
                 self.adjoint_sensitivity_eqs_picard[i] -= self.w_1[i]
 
         self.adjoint_sensitivity_eqs_rhs = [
-            summation(
+            _utils.summation(
                 [
                     fenics.derivative(
                         self.adjoint_sensitivity_eqs_all_temp[j],
@@ -652,6 +557,39 @@ class ControlFormHandler(FormHandler):
             for i in range(self.control_dim)
         ]
 
+    def compute_newton_forms(self) -> None:
+        """Calculates the needed forms for the truncated Newton method."""
+        if self.use_scalar_tracking or self.use_min_max_terms:
+            raise _exceptions.InputError(
+                "cashocs._forms.ShapeFormHandler",
+                "_compute_newton_forms",
+                (
+                    "Newton's method is not available with scalar tracking or"
+                    " min_max terms."
+                ),
+            )
+
+        self._compute_sensitivity_equations()
+        self._compute_first_order_lagrangian_derivatives()
+        self._compute_second_order_lagrangian_derivatives()
+
+        self.w_1 = [
+            _utils.summation([self.lagrangian_yy[i][j] for j in range(self.state_dim)])
+            + _utils.summation(
+                [self.lagrangian_uy[i][j] for j in range(self.control_dim)]
+            )
+            for i in range(self.state_dim)
+        ]
+        self.w_2 = [
+            _utils.summation([self.lagrangian_yu[i][j] for j in range(self.state_dim)])
+            + _utils.summation(
+                [self.lagrangian_uu[i][j] for j in range(self.control_dim)]
+            )
+            for i in range(self.control_dim)
+        ]
+
+        self._compute_adjoint_sensitivity_equations()
+
         self.w_3 = [
             -self.adjoint_sensitivity_eqs_rhs[i] for i in range(self.control_dim)
         ]
