@@ -1,68 +1,65 @@
-# Copyright (C) 2020-2021 Sebastian Blauth
+# Copyright (C) 2020-2022 Sebastian Blauth
 #
-# This file is part of CASHOCS.
+# This file is part of cashocs.
 #
-# CASHOCS is free software: you can redistribute it and/or modify
+# cashocs is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
-# CASHOCS is distributed in the hope that it will be useful,
+# cashocs is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with CASHOCS.  If not, see <https://www.gnu.org/licenses/>.
+# along with cashocs.  If not, see <https://www.gnu.org/licenses/>.
 
 """Abstract implementation of a shape gradient problem.
 
-This class uses the linear elasticity equations to project the
-shape derivative to the shape gradient with a Riesz projection.
+This class uses the linear elasticity equations to project the shape derivative to the
+shape gradient with a Riesz projection.
 """
 
 from __future__ import annotations
 
 import configparser
-from typing import TYPE_CHECKING, List
+from typing import List, TYPE_CHECKING
 
 import fenics
 import numpy as np
-import ufl
 from petsc4py import PETSc
+import ufl
 
-from .._interfaces.pde_problem import PDEProblem
-from ..nonlinear_solvers import newton_solve
-from ..utils import _setup_petsc_options, _solve_linear_problem
-
+from cashocs import _loggers
+from cashocs import _utils
+from cashocs import nonlinear_solvers
+from cashocs._pde_problems import pde_problem
 
 if TYPE_CHECKING:
-    from .state_problem import StateProblem
-    from .adjoint_problem import AdjointProblem
-    from .._forms import ShapeFormHandler
+    from cashocs import _forms
+    from cashocs._pde_problems import adjoint_problem as ap
+    from cashocs._pde_problems import state_problem as sp
 
 
-class ShapeGradientProblem(PDEProblem):
+class ShapeGradientProblem(pde_problem.PDEProblem):
     """Riesz problem for the computation of the shape gradient."""
 
     def __init__(
         self,
-        form_handler: ShapeFormHandler,
-        state_problem: StateProblem,
-        adjoint_problem: AdjointProblem,
+        form_handler: _forms.ShapeFormHandler,
+        state_problem: sp.StateProblem,
+        adjoint_problem: ap.AdjointProblem,
     ) -> None:
-        """Initialize the ShapeGradientProblem.
+        """Initializes self.
+
+        Args:
+            form_handler: The ShapeFormHandler object corresponding to the shape
+                optimization problem.
+            state_problem: The corresponding state problem.
+            adjoint_problem: The corresponding adjoint problem.
 
-        Parameters
-        ----------
-        form_handler : ShapeFormHandler
-            The ShapeFormHandler object corresponding to the shape optimization problem.
-        state_problem : StateProblem
-            The corresponding state problem.
-        adjoint_problem : AdjointProblem
-            The corresponding adjoint problem.
         """
-
         super().__init__(form_handler)
         self.state_problem = state_problem
         self.adjoint_problem = adjoint_problem
@@ -70,25 +67,22 @@ class ShapeGradientProblem(PDEProblem):
         self.gradient = self.form_handler.gradient
         self.gradient_norm_squared = 1.0
 
-        gradient_tol = self.config.getfloat(
-            "OptimizationRoutine", "gradient_tol", fallback=1e-9
-        )
+        gradient_tol = self.config.getfloat("OptimizationRoutine", "gradient_tol")
 
         # Generate the Krylov solver for the shape gradient problem
+        # noinspection PyUnresolvedReferences
         self.ksp = PETSc.KSP().create()
 
-        gradient_method = self.config.get(
-            "OptimizationRoutine", "gradient_method", fallback="direct"
-        )
+        gradient_method = self.config.get("OptimizationRoutine", "gradient_method")
 
-        if gradient_method == "direct":
+        if gradient_method.casefold() == "direct":
             self.ksp_options = [
                 ["ksp_type", "preonly"],
                 ["pc_type", "lu"],
                 ["pc_factor_mat_solver_type", "mumps"],
                 ["mat_mumps_icntl_24", 1],
             ]
-        elif gradient_method == "iterative":
+        elif gradient_method.casefold() == "iterative":
             self.ksp_options = [
                 ["ksp_type", "cg"],
                 ["pc_type", "hypre"],
@@ -99,13 +93,24 @@ class ShapeGradientProblem(PDEProblem):
                 ["ksp_max_it", 250],
             ]
 
-        _setup_petsc_options([self.ksp], [self.ksp_options])
+        _utils.setup_petsc_options([self.ksp], [self.ksp_options])
 
         if (
-            self.config.getboolean("ShapeGradient", "use_p_laplacian", fallback=False)
-            and not self.form_handler.uses_custom_scalar_product
+            self.config.getboolean("ShapeGradient", "use_p_laplacian")
+            and self.form_handler.use_fixed_dimensions
         ):
-            self.p_laplace_projector = _PLaplacProjector(
+            _loggers.warning(
+                "Incompatible config settings: "
+                "use_p_laplacian and fixed_dimensions are incompatible. "
+                "Falling back to use_p_laplacian=False."
+            )
+
+        if (
+            self.config.getboolean("ShapeGradient", "use_p_laplacian")
+            and not self.form_handler.uses_custom_scalar_product
+            and not self.form_handler.use_fixed_dimensions
+        ):
+            self.p_laplace_projector = _PLaplaceProjector(
                 self,
                 self.gradient,
                 self.form_handler.shape_derivative,
@@ -113,15 +118,14 @@ class ShapeGradientProblem(PDEProblem):
                 self.config,
             )
 
-    def solve(self) -> fenics.Function:
-        """Solves the Riesz projection problem to obtain the shape gradient of the cost functional.
+    def solve(self) -> List[fenics.Function]:
+        """Solves the Riesz projection problem to obtain the shape gradient.
+
+        Returns:
+            The function representing the shape gradient of the (reduced) cost
+            functional.
 
-        Returns
-        -------
-        fenics.Function
-            The function representing the shape gradient of the (reduced) cost functional.
         """
-
         self.state_problem.solve()
         self.adjoint_problem.solve()
 
@@ -130,10 +134,9 @@ class ShapeGradientProblem(PDEProblem):
             self.form_handler.shape_regularization.update_geometric_quantities()
 
             if (
-                self.config.getboolean(
-                    "ShapeGradient", "use_p_laplacian", fallback=False
-                )
+                self.config.getboolean("ShapeGradient", "use_p_laplacian")
                 and not self.form_handler.uses_custom_scalar_product
+                and not self.form_handler.use_fixed_dimensions
             ):
                 self.p_laplace_projector.solve()
                 self.has_solution = True
@@ -150,7 +153,7 @@ class ShapeGradientProblem(PDEProblem):
                     self.form_handler.fe_shape_derivative_vector[
                         self.form_handler.fixed_indices
                     ] = 0.0
-                _solve_linear_problem(
+                _utils.solve_linear_problem(
                     self.ksp,
                     self.form_handler.scalar_product_matrix,
                     self.form_handler.fe_shape_derivative_vector.vec(),
@@ -165,13 +168,13 @@ class ShapeGradientProblem(PDEProblem):
                     self.gradient, self.gradient
                 )
 
-            self.form_handler._post_hook()
+            self.form_handler.post_hook()
 
         return self.gradient
 
 
-class _PLaplacProjector:
-    """A class for computing the gradient deformation with a p-Laplace projection"""
+class _PLaplaceProjector:
+    """A class for computing the gradient deformation with a p-Laplace projection."""
 
     def __init__(
         self,
@@ -181,27 +184,19 @@ class _PLaplacProjector:
         bcs_shape: List[fenics.DirichletBC],
         config: configparser.ConfigParser,
     ) -> None:
-        """
+        """Initializes self.
 
-        Parameters
-        ----------
-        gradient_problem : ShapeGradientProblem
-            The shape gradient problem
-        gradient : list[fenics.Function]
-            The fenics Function representing the gradient deformation
-        shape_derivative : ufl.Form
-            The ufl Form of the shape derivative
-        bcs_shape : list[fenics.DirichletBC]
-            The boundary conditions for computing the gradient deformation
-        config : configparser.ConfigParser
-            The config for the optimization problem
-        """
+        Args:
+            gradient_problem: The shape gradient problem
+            gradient: The fenics Function representing the gradient deformation
+            shape_derivative: The ufl Form of the shape derivative
+            bcs_shape: The boundary conditions for computing the gradient deformation
+            config: The config for the optimization problem
 
-        self.p_target = config.getint("ShapeGradient", "p_laplacian_power", fallback=2)
-        delta = config.getfloat("ShapeGradient", "damping_factor", fallback=0.0)
-        eps = config.getfloat(
-            "ShapeGradient", "p_laplacian_stabilization", fallback=0.0
-        )
+        """
+        self.p_target = config.getint("ShapeGradient", "p_laplacian_power")
+        delta = config.getfloat("ShapeGradient", "damping_factor")
+        eps = config.getfloat("ShapeGradient", "p_laplacian_stabilization")
         self.p_list = np.arange(2, self.p_target + 1, 1)
         self.solution = gradient[0]
         self.shape_derivative = shape_derivative
@@ -210,16 +205,16 @@ class _PLaplacProjector:
         dx = gradient_problem.form_handler.dx
         self.mu_lame = gradient_problem.form_handler.mu_lame
 
-        self.A_tensor = fenics.PETScMatrix()
+        self.A_tensor = fenics.PETScMatrix()  # pylint: disable=invalid-name
         self.b_tensor = fenics.PETScVector()
 
-        self.F_list = []
+        self.form_list = []
         for p in self.p_list:
             kappa = pow(
                 fenics.inner(fenics.grad(self.solution), fenics.grad(self.solution)),
                 (p - 2) / 2.0,
             )
-            self.F_list.append(
+            self.form_list.append(
                 fenics.inner(
                     self.mu_lame
                     * (fenics.Constant(eps) + kappa)
@@ -232,18 +227,16 @@ class _PLaplacProjector:
                 * dx
             )
 
-            gradient_method = config.get(
-                "OptimizationRoutine", "gradient_method", fallback="direct"
-            )
+            gradient_method = config.get("OptimizationRoutine", "gradient_method")
 
-            if gradient_method == "direct":
+            if gradient_method.casefold() == "direct":
                 self.ksp_options = [
                     ["ksp_type", "preonly"],
                     ["pc_type", "lu"],
                     ["pc_factor_mat_solver_type", "mumps"],
                     ["mat_mumps_icntl_24", 1],
                 ]
-            elif gradient_method == "iterative":
+            elif gradient_method.casefold() == "iterative":
                 self.ksp_options = [
                     ["ksp_type", "cg"],
                     ["pc_type", "hypre"],
@@ -254,17 +247,11 @@ class _PLaplacProjector:
                 ]
 
     def solve(self) -> None:
-        """Solves the p-Laplace problem for computing the shape gradient
-
-        Returns
-        -------
-        None
-        """
-
+        """Solves the p-Laplace problem for computing the shape gradient."""
         self.solution.vector().vec().set(0.0)
-        for F in self.F_list:
-            newton_solve(
-                F,
+        for nonlinear_form in self.form_list:
+            nonlinear_solvers.newton_solve(
+                nonlinear_form,
                 self.solution,
                 self.bcs_shape,
                 shift=self.shape_derivative,
